@@ -49,6 +49,7 @@ import json
 import tempfile
 import logging
 
+import mamba
 import mamba.mamba_api as api
 
 from mamba.post_solve_handling import post_solve_handling
@@ -57,7 +58,7 @@ from mamba.utils import get_index, to_package_record_from_subjson, init_api_cont
 log = getLogger(__name__)
 stderrlog = getLogger('mamba.stderr')
 
-banner = """
+banner = f"""
                   __    __    __    __
                  /  \\  /  \\  /  \\  /  \\
                 /    \\/    \\/    \\/    \\
@@ -73,7 +74,7 @@ banner = """
         ██║ ╚═╝ ██║██║  ██║██║ ╚═╝ ██║██████╔╝██║  ██║
         ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝     ╚═╝╚═════╝ ╚═╝  ╚═╝
 
-        Supported by @QuantStack
+        mamba ({mamba.__version__}) supported by @QuantStack
 
         GitHub:  https://github.com/QuantStack/mamba
         Twitter: https://twitter.com/QuantStack
@@ -183,13 +184,12 @@ def handle_txn(unlink_link_transaction, prefix, args, newenv, remove_op=False):
         elif not newenv:
             if context.json:
                 cli_common.stdout_json_success(message='All requested packages already installed.')
-            else:
-                print('\n# All requested packages already installed.\n')
             return
 
     if context.dry_run:
         actions = unlink_link_transaction._make_legacy_action_groups()[0]
-        cli_common.stdout_json_success(prefix=prefix, actions=actions, dry_run=True)
+        if context.json:
+            cli_common.stdout_json_success(prefix=prefix, actions=actions, dry_run=True)
         raise DryRunExit()
 
     try:
@@ -271,8 +271,6 @@ def remove(args, parser):
             specs = [s for s in specs_from_args(args.package_names)]
         if not context.quiet:
             print("Removing specs: {}".format([s.conda_build_form() for s in specs]))
-        channel_urls = ()
-        subdirs = ()
 
         installed_json_f = get_installed_jsonfile(prefix)
 
@@ -302,7 +300,8 @@ def remove(args, parser):
             exit_code = 1
             return exit_code
 
-        transaction = api.Transaction(solver)
+        package_cache = api.MultiPackageCache(context.pkgs_dirs)
+        transaction = api.Transaction(solver, package_cache)
         to_link, to_unlink = transaction.to_conda()
         transaction.log_json()
 
@@ -316,7 +315,7 @@ def install(args, parser, command='install'):
     context.validate_configuration()
     check_non_admin()
 
-    init_api_context()
+    init_api_context(use_mamba_experimental)
 
     newenv = bool(command == 'create')
     isinstall = bool(command == 'install')
@@ -362,7 +361,6 @@ def install(args, parser, command='install'):
             else:
                 raise EnvironmentLocationNotFound(prefix)
 
-    prepend = not args.override_channels
     prefix = context.target_prefix
 
     index_args = {
@@ -510,7 +508,6 @@ def install(args, parser, command='install'):
         repo.set_installed()
         repos.append(repo)
 
-
     for channel, cache_file, priority, subpriority in channel_json:
         repo = api.Repo(pool, str(channel), cache_file, channel.url(with_credentials=True))
         repo.set_priority(priority, subpriority)
@@ -524,7 +521,8 @@ def install(args, parser, command='install'):
         exit_code = 1
         return exit_code
 
-    transaction = api.Transaction(solver)
+    package_cache = api.MultiPackageCache(context.pkgs_dirs)
+    transaction = api.Transaction(solver, package_cache)
     to_link, to_unlink = transaction.to_conda()
     transaction.log_json()
 
@@ -536,8 +534,13 @@ def install(args, parser, command='install'):
     if python_added:
         specs = [s for s in specs if s.name != 'python']
 
-    conda_transaction = to_txn(specs, (), prefix, to_link, to_unlink, index)
-    handle_txn(conda_transaction, prefix, args, newenv)
+    if use_mamba_experimental and not os.name == 'nt':
+        if command == 'create' and not isdir(context.target_prefix):
+            mkdir_p(prefix)
+        transaction.execute(prefix_data, PackageCacheData.first_writable().pkgs_dir)
+    else:
+        conda_transaction = to_txn(specs, (), prefix, to_link, to_unlink, index)
+        handle_txn(conda_transaction, prefix, args, newenv)
 
     try:
         installed_json_f.close()
@@ -575,7 +578,6 @@ def update(args, parser):
     install(args, parser, 'update')
 
 def repoquery(args, parser):
-    prepend = not args.override_channels
     prefix = context.target_prefix
 
     init_api_context()
@@ -588,11 +590,6 @@ def repoquery(args, parser):
         'use_local': args.use_local
     }
 
-    index = get_index(channel_urls=index_args['channel_urls'],
-                  prepend=index_args['prepend'], platform=None,
-                  use_local=index_args['use_local'], use_cache=index_args['use_cache'],
-                  unknown=index_args['unknown'], prefix=prefix)
-
     installed_json_f = get_installed_jsonfile(prefix)
 
     pool = api.Pool()
@@ -603,20 +600,35 @@ def repoquery(args, parser):
     repo.set_installed()
     repos.append(repo)
 
-    if not args.installed:
+    only_installed = True
+    if args.subcmd == "search" and args.installed == False:
+        only_installed = False
+
+    if not only_installed:
+        index = get_index(channel_urls=index_args['channel_urls'],
+                      prepend=index_args['prepend'], platform=None,
+                      use_local=index_args['use_local'], use_cache=index_args['use_cache'],
+                      unknown=index_args['unknown'], prefix=prefix)
+
         for subdir, channel in index:
+            if subdir.loaded() == False and channel.platform != 'noarch':
+                # ignore non-loaded subdir if channel is != noarch
+                continue
+
             repo = api.Repo(pool, str(channel), subdir.cache_path(), channel.url(with_credentials=True))
             repo.set_priority(0, 0)
             repos.append(repo)
 
 
-    print("\nExecuting the query %s\n" % args.query)
+    print("\nExecuting the query %s\n" % args.package_query)
 
     query = api.Query(pool)
-    if args.whatrequires:
-        print(query.whatrequires(args.query))
-    else:
-        print(query.find(args.query))
+    if args.subcmd == "whoneeds":
+        print(query.whatrequires(args.package_query, args.tree))
+    if args.subcmd == "depends":
+        print(query.dependencytree(args.package_query))
+    if args.subcmd == "search":
+        print(query.find(args.package_query))
 
 
 def do_call(args, parser):
@@ -650,36 +662,49 @@ def configure_parser_repoquery(sub_parsers):
     example = ("""
 Examples:
 
-    conda repoquery xtensor>=0.18
+    mamba repoquery search xtensor>=0.18
+    mamba repoquery depends xtensor
+    mamba repoquery whoneeds xtl
 
     """)
 
+    import argparse
     from argparse import SUPPRESS
 
     p = sub_parsers.add_parser(
         'repoquery',
         description=descr,
         help=help,
-        epilog=example,
+        epilog=example
     )
-
-    p.add_argument(
-        'query',
-        action="store",
-        nargs='?',
-        help="Package query.",
-    )
-
-    p.add_argument(
-        "--whatrequires",
+    subsub_parser = p.add_subparsers(dest='subcmd')
+    package_cmds = argparse.ArgumentParser(add_help=False)
+    package_cmds.add_argument('package_query', help='the target package')
+    package_cmds.add_argument(
+        "-i", "--installed",
         action="store_true",
-        help=SUPPRESS,
+        help=SUPPRESS
     )
 
-    p.add_argument(
-        "--installed",
-        action="store_true",
-        help=SUPPRESS,
+    view_cmds = argparse.ArgumentParser(add_help=False)
+    view_cmds.add_argument(
+        "-t", "--tree",
+        action="store_true"
+    )
+
+    subsub_parser.add_parser('whoneeds',
+        help='shows packages that depends on this package',
+        parents=[package_cmds, view_cmds]
+    )
+
+    subsub_parser.add_parser('depends',
+        help='shows packages that depends on this package',
+        parents=[package_cmds, view_cmds]
+    )
+
+    subsub_parser.add_parser('search',
+        help='shows packages that depends on this package',
+        parents=[package_cmds]
     )
 
     from conda.cli import conda_argparse
@@ -690,7 +715,6 @@ Examples:
     p.set_defaults(func='.main_repoquery.execute')
     return p
 
-
 def _wrapped_main(*args, **kwargs):
     if len(args) == 1:
         args = args + ('-h',)
@@ -735,6 +759,10 @@ def main(*args, **kwargs):
     if not args:
         args = sys.argv
 
+    if "--version" in args:
+        from mamba._version import __version__
+        print("mamba {}".format(__version__))
+
     args = tuple(ensure_text_type(s) for s in args)
 
     if len(args) > 2 and args[1] == 'env' and args[2] == 'create':
